@@ -1,3 +1,4 @@
+/* $Id: ClpDualRowSteepest.cpp 1458 2009-11-05 12:34:07Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 
@@ -228,7 +229,7 @@ ClpDualRowSteepest::pivotRow()
   // But cap
   tolerance = CoinMin(1000.0,tolerance);
   tolerance *= tolerance; // as we are using squares
-  double saveTolerance = tolerance;
+  bool toleranceChanged = false;
   double * solution = model_->solutionRegion();
   double * lower = model_->lowerRegion();
   double * upper = model_->upperRegion();
@@ -236,14 +237,6 @@ ClpDualRowSteepest::pivotRow()
   //#define COLUMN_BIAS 4.0
   //#define FIXED_BIAS 10.0
   if (lastPivotRow>=0&&lastPivotRow<model_->numberRows()) {
-#if defined (__MINGW32__) || defined(__CYGWIN32__)
-    if (model_->numberIterations()<0)
-      printf("aab_p it %d\n",model_->numberIterations());
-#endif
-#if defined (__MINGW32__) || defined(__CYGWIN32__)
-    if (model_->numberIterations()<0)
-      printf("aab_p it %d\n",model_->numberIterations());
-#endif
 #ifdef COLUMN_BIAS 
     int numberColumns = model_->numberColumns();
 #endif
@@ -283,14 +276,12 @@ k
     }
     number = infeasible_->getNumElements();
   }
-#if defined(__MINGW32__) || defined(__CYGWIN32__)
-  if (model_->numberIterations()<0)
-    printf("aac_p it %d\n",model_->numberIterations());
-#endif
   if(model_->numberIterations()<model_->lastBadIteration()+200) {
     // we can't really trust infeasibilities if there is dual error
-    if (model_->largestDualError()>model_->largestPrimalError())
+    if (model_->largestDualError()>model_->largestPrimalError()) {
       tolerance *= CoinMin(model_->largestDualError()/model_->largestPrimalError(),1000.0);
+      toleranceChanged=true;
+    }
   }
   int numberWanted;
   if (mode_<2 ) {
@@ -299,7 +290,8 @@ k
     numberWanted = CoinMax(2000,number/8);
   } else {
     int numberElements = model_->factorization()->numberElements();
-    double ratio = (double) numberElements/(double) model_->numberRows();
+    double ratio = static_cast<double> (numberElements)/
+      static_cast<double> (model_->numberRows());
     numberWanted = CoinMax(2000,number/8);
     if (ratio<1.0) {
       numberWanted = CoinMax(2000,number/20);
@@ -308,7 +300,7 @@ k
       if (ratio>number)
 	numberWanted=number+1;
       else
-	numberWanted = CoinMax(2000,(int) ratio);
+	numberWanted = CoinMax(2000,static_cast<int> (ratio));
     }
   }
   if (model_->largestPrimalError()>1.0e-3)
@@ -318,8 +310,8 @@ k
   int start[4];
   start[1]=number;
   start[2]=0;
-  double dstart = ((double) number) * model_->randomNumberGenerator()->randomDouble();
-  start[0]=(int) dstart;
+  double dstart = static_cast<double> (number) * model_->randomNumberGenerator()->randomDouble();
+  start[0]=static_cast<int> (dstart);
   start[3]=start[0];
   //double largestWeight=0.0;
   //double smallestWeight=1.0e100;
@@ -382,7 +374,7 @@ k
       break;
   }
   //printf("smallest %g largest %g\n",smallestWeight,largestWeight);
-  if (chosenRow<0&& tolerance>saveTolerance) {
+  if (chosenRow<0 && toleranceChanged) {
     // won't line up with checkPrimalSolution - do again
     double saveError = model_->largestDualError();
     model_->setLargestDualError(0.0);
@@ -390,6 +382,19 @@ k
     chosenRow=pivotRow();
     model_->setLargestDualError(saveError);
   }
+  if (chosenRow<0&&lastPivotRow<0){
+    int nLeft=0;
+    for (int i=0;i<number;i++) {
+      int iRow = index[i];
+      if (fabs(infeas[iRow])>1.0e-50) {
+	index[nLeft++]=iRow;
+      } else {
+	infeas[iRow]=0.0;
+      }
+    }
+    infeasible_->setNumElements(nLeft);
+    model_->setNumberPrimalInfeasibilities(nLeft);
+  }
   return chosenRow;
 }
 #if 0
@@ -495,13 +500,13 @@ ClpDualRowSteepest::updateWeights(CoinIndexedVector * input,
     ft_count_in+= updatedColumn->getNumElements();
     up_count_in+= spare->getNumElements();
 #endif
-    if (permute) {
+    if (permute||true) {
 #if CLP_DEBUG>2
       printf("REGION before %d els\n",spare->getNumElements());
       spare->print();
 #endif
       model_->factorization()->updateTwoColumnsFT(spare2,updatedColumn,
-						  spare,true);
+						  spare,permute!=NULL);
 #if CLP_DEBUG>2
       printf("REGION after %d els\n",spare->getNumElements());
       spare->print();
@@ -531,7 +536,7 @@ ClpDualRowSteepest::updateWeights(CoinIndexedVector * input,
 #endif
     // could re-initialize here (could be expensive)
     norm /= model_->alpha() * model_->alpha();
-    
+    assert(model_->alpha());
     assert(norm);
     // pivot element
     alpha=0.0;
@@ -666,16 +671,18 @@ ClpDualRowSteepest::updatePrimalSolution(
 					double primalRatio,
 					double & objectiveChange)
 {
-  double * work = primalUpdate->denseVector();
+  double * COIN_RESTRICT work = primalUpdate->denseVector();
   int number = primalUpdate->getNumElements();
-  int * which = primalUpdate->getIndices();
+  int * COIN_RESTRICT which = primalUpdate->getIndices();
   int i;
   double changeObj=0.0;
   double tolerance=model_->currentPrimalTolerance();
-  const int * pivotVariable = model_->pivotVariable();
-  double * infeas = infeasible_->denseVector();
-  int pivotRow = model_->pivotRow();
-  double * solution = model_->solutionRegion();
+  const int * COIN_RESTRICT pivotVariable = model_->pivotVariable();
+  double * COIN_RESTRICT infeas = infeasible_->denseVector();
+  double * COIN_RESTRICT solution = model_->solutionRegion();
+  const double * COIN_RESTRICT costModel = model_->costRegion();
+  const double * COIN_RESTRICT lowerModel = model_->lowerRegion();
+  const double * COIN_RESTRICT upperModel = model_->upperRegion();
 #ifdef COLUMN_BIAS 
   int numberColumns = model_->numberColumns();
 #endif
@@ -684,23 +691,14 @@ ClpDualRowSteepest::updatePrimalSolution(
       int iRow=which[i];
       int iPivot=pivotVariable[iRow];
       double value = solution[iPivot];
-      double cost = model_->cost(iPivot);
+      double cost = costModel[iPivot];
       double change = primalRatio*work[i];
       work[i]=0.0;
       value -= change;
       changeObj -= change*cost;
+      double lower = lowerModel[iPivot];
+      double upper = upperModel[iPivot];
       solution[iPivot] = value;
-      double lower = model_->lower(iPivot);
-      double upper = model_->upper(iPivot);
-      // But if pivot row then use value of incoming
-      // Although it is safer to recompute before next selection
-      // in case something odd happens
-      if (iRow==pivotRow) {
-	iPivot = model_->sequenceIn();
-	lower = model_->lower(iPivot);
-	upper = model_->upper(iPivot);
-	value = model_->valueIncomingDual();
-      }
       if (value<lower-tolerance) {
 	value -= lower;
 	value *= value;
@@ -744,22 +742,13 @@ ClpDualRowSteepest::updatePrimalSolution(
       int iRow=which[i];
       int iPivot=pivotVariable[iRow];
       double value = solution[iPivot];
-      double cost = model_->cost(iPivot);
+      double cost = costModel[iPivot];
       double change = primalRatio*work[iRow];
       value -= change;
       changeObj -= change*cost;
+      double lower = lowerModel[iPivot];
+      double upper = upperModel[iPivot];
       solution[iPivot] = value;
-      double lower = model_->lower(iPivot);
-      double upper = model_->upper(iPivot);
-      // But if pivot row then use value of incoming
-      // Although it is safer to recompute before next selection
-      // in case something odd happens
-      if (iRow==pivotRow) {
-	iPivot = model_->sequenceIn();
-	lower = model_->lower(iPivot);
-	upper = model_->upper(iPivot);
-	value = model_->valueIncomingDual();
-      }
       if (value<lower-tolerance) {
 	value -= lower;
 	value *= value;
@@ -800,6 +789,14 @@ ClpDualRowSteepest::updatePrimalSolution(
       work[iRow]=0.0;
     }
   }
+  // Do pivot row
+  {
+    int iRow = model_->pivotRow();
+    // feasible - was it infeasible - if so set tiny
+    //assert (infeas[iRow]);
+    if (infeas[iRow])
+      infeas[iRow] = COIN_INDEXED_REALLY_TINY_ELEMENT;
+  }
   primalUpdate->setNumElements(0);
   objectiveChange += changeObj;
 }
@@ -845,7 +842,7 @@ ClpDualRowSteepest::saveWeights(ClpSimplex * model,int mode)
 	state_=-1;
       }
     }
-  } else if (mode==2||mode==4||mode==5) {
+  } else if (mode==2||mode==4||mode>=5) {
     // restore
     if (!weights_||state_==-1||mode==5) {
       // initialize weights
@@ -896,7 +893,7 @@ ClpDualRowSteepest::saveWeights(ClpSimplex * model,int mode)
 	array[i]=weights_[i];
 	which[i]=pivotVariable[i];
       }
-    } else {
+    } else if (mode!=6) {
       int * which = alternateWeights_->getIndices();
       CoinIndexedVector * rowArray3 = model_->rowArray(3);
       rowArray3->clear();
@@ -906,8 +903,8 @@ ClpDualRowSteepest::saveWeights(ClpSimplex * model,int mode)
 	back[i]=-1;
       if (mode!=4) {
 	// save
- CoinMemcpyN(which,	numberRows,savedWeights_->getIndices());
- CoinMemcpyN(weights_,	numberRows,savedWeights_->denseVector());
+	CoinMemcpyN(which,	numberRows,savedWeights_->getIndices());
+	CoinMemcpyN(weights_,	numberRows,savedWeights_->denseVector());
       } else {
 	// restore
 	//memcpy(which,savedWeights_->getIndices(),
@@ -934,6 +931,27 @@ ClpDualRowSteepest::saveWeights(ClpSimplex * model,int mode)
 	  weights_[i]=1.0;
 	}
       }
+    } else {
+      // mode 6 - scale back weights as primal errors
+      double primalError = model_->largestPrimalError();
+      double allowed ;
+      if (primalError>1.0e3)
+	allowed=10.0;
+      else if (primalError>1.0e2)
+	allowed=50.0;
+      else if (primalError>1.0e1)
+	allowed=100.0;
+      else
+	allowed=1000.0;
+      double allowedInv =1.0/allowed;
+      for (i=0;i<numberRows;i++) {
+	double value = weights_[i];
+	if (value<allowedInv)
+	  value = allowedInv;
+	else if (value>allowed)
+	  value=allowed;
+	weights_[i]=allowed;
+      }
     }
     state_=0;
     // set up infeasibilities
