@@ -1,3 +1,4 @@
+/* $Id: ClpFactorization.hpp 1458 2009-11-05 12:34:07Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #ifndef ClpFactorization_H
@@ -10,18 +11,15 @@
 class ClpMatrixBase;
 class ClpSimplex;
 class ClpNetworkBasis;
+class CoinOtherFactorization;
 #ifndef CLP_MULTIPLE_FACTORIZATIONS 
-#define CLP_MULTIPLE_FACTORIZATIONS 3
+#define CLP_MULTIPLE_FACTORIZATIONS 4
 #endif    
-#if CLP_MULTIPLE_FACTORIZATIONS == 1
+#ifdef CLP_MULTIPLE_FACTORIZATIONS 
 #include "CoinDenseFactorization.hpp"
-typedef CoinDenseFactorization CoinSmallFactorization;
-#elif CLP_MULTIPLE_FACTORIZATIONS == 2
-#include "CoinSimpFactorization.hpp"
-typedef CoinSimpFactorization CoinSmallFactorization;
-#elif CLP_MULTIPLE_FACTORIZATIONS == 3
-#include "CoinDenseFactorization.hpp"
-#include "CoinSimpFactorization.hpp"
+#endif
+#ifndef COIN_FAST_CODE
+#define COIN_FAST_CODE
 #endif
 
 /** This just implements CoinFactorization when an ClpMatrixBase object
@@ -67,8 +65,8 @@ public:
    /** The copy constructor. */
   ClpFactorization(const ClpFactorization&,int denseIfSmaller=0);
 #ifdef CLP_MULTIPLE_FACTORIZATIONS    
-   /** The copy constructor from an CoinSmallFactorization. */
-   ClpFactorization(const CoinSmallFactorization&);
+   /** The copy constructor from an CoinOtherFactorization. */
+   ClpFactorization(const CoinOtherFactorization&);
 #endif
    ClpFactorization& operator=(const ClpFactorization&);
    //@}
@@ -89,7 +87,8 @@ public:
 		      CoinIndexedVector * tableauColumn,
 		      int pivotRow,
 		      double pivotCheck ,
-		      bool checkBeforeModifying=false);
+		      bool checkBeforeModifying=false,
+		      double acceptablePivot=1.0e-8);
   //@}
 
   /**@name various uses of factorization (return code number elements) 
@@ -213,7 +212,7 @@ public:
   }
   /// Level of detail of messages
   inline int messageLevel (  ) const {
-    if (coinFactorizationA_) return coinFactorizationA_->messageLevel();  else return 0 ;
+    if (coinFactorizationA_) return coinFactorizationA_->messageLevel();  else return 1 ;
   }
   /// Set level of detail of messages
   inline void messageLevel (  int value) {
@@ -221,7 +220,11 @@ public:
   }
   /// Get rid of all memory
   inline void clearArrays()
-  { if (coinFactorizationA_) coinFactorizationA_->clearArrays();}
+  { if (coinFactorizationA_) 
+      coinFactorizationA_->clearArrays();
+    else if (coinFactorizationB_) 
+      coinFactorizationB_->clearArrays();
+  }
   /// Number of Rows after factorization
   inline int numberRows (  ) const {
     if (coinFactorizationA_) return coinFactorizationA_->numberRows(); else return coinFactorizationB_->numberRows() ;
@@ -234,11 +237,12 @@ public:
   { if (coinFactorizationA_) coinFactorizationA_->setDenseThreshold(value);}
   /// Pivot tolerance
   inline double pivotTolerance (  ) const {
-    if (coinFactorizationA_) return coinFactorizationA_->pivotTolerance();  else return 1.0e-8 ;
+    if (coinFactorizationA_) return coinFactorizationA_->pivotTolerance();  else if (coinFactorizationB_) return coinFactorizationB_->pivotTolerance();return 1.0e-8 ;
   }
   /// Set pivot tolerance
   inline void pivotTolerance (  double value) {
     if (coinFactorizationA_) coinFactorizationA_->pivotTolerance(value);
+    else if (coinFactorizationB_) coinFactorizationB_->pivotTolerance(value);
   }
   /// Allows change of pivot accuracy check 1.0 == none >1.0 relaxed
   inline void relaxAccuracyCheck(double value)
@@ -254,7 +258,11 @@ public:
   { if (coinFactorizationA_) coinFactorizationA_->setPersistenceFlag(value);}
   /// Delete all stuff (leaves as after CoinFactorization())
   inline void almostDestructor()
-  { if (coinFactorizationA_) coinFactorizationA_->almostDestructor(); }
+  { if (coinFactorizationA_) 
+      coinFactorizationA_->almostDestructor();
+    else if (coinFactorizationB_) 
+      coinFactorizationB_->clearArrays();
+  }
   /// Returns areaFactor but adjusted for dense
   inline double adjustedAreaFactor() const
   { if (coinFactorizationA_) return coinFactorizationA_->adjustedAreaFactor(); else return 0.0 ; }
@@ -273,6 +281,14 @@ public:
       coinFactorizationA_->zeroTolerance(1.0e-13);
     }
   }
+  /// If nonzero force use of 1,dense 2,small 3,osl 
+  void forceOtherFactorization(int which);
+  /// Get switch to osl if number rows <= this
+  inline int goOslThreshold() const
+  { return goOslThreshold_;}
+  /// Set switch to osl if number rows <= this
+  inline void setGoOslThreshold(int value)
+  { goOslThreshold_ = value;}
   /// Get switch to dense if number rows <= this
   inline int goDenseThreshold() const
   { return goDenseThreshold_;}
@@ -287,6 +303,8 @@ public:
   { goSmallThreshold_ = value;}
   /// Go over to dense or small code if small enough
   void goDenseOrSmall(int numberRows) ;
+  /// Sets factorization
+  void setFactorization(ClpFactorization & factorization);
   /// Return 1 if dense code
   inline int isDenseOrSmall() const
   { return coinFactorizationB_ ? 1 : 0;}
@@ -343,8 +361,12 @@ private:
 #ifdef CLP_MULTIPLE_FACTORIZATIONS    
   /// Pointer to CoinFactorization 
   CoinFactorization * coinFactorizationA_;
-  /// Pointer to CoinSmallFactorization 
-  CoinSmallFactorization * coinFactorizationB_;
+  /// Pointer to CoinOtherFactorization 
+  CoinOtherFactorization * coinFactorizationB_;
+  /// If nonzero force use of 1,dense 2,small 3,osl 
+  int forceB_;
+  /// Switch to osl if number rows <= this
+  int goOslThreshold_;
   /// Switch to small if number rows <= this
   int goSmallThreshold_;
   /// Switch to dense if number rows <= this
