@@ -1,3 +1,4 @@
+/* $Id: ClpNode.cpp 1430 2009-09-15 15:51:17Z forrest $ */
 // Copyright (C) 2008, International Business Machines
 // Corporation and others.  All Rights Reserved.
 
@@ -30,6 +31,7 @@ ClpNode::ClpNode () :
   fixed_(NULL),
   sequence_(1),
   numberInfeasibilities_(0),
+  depth_(0),
   numberFixed_(0),
   flags_(0),
   maximumFixed_(0),
@@ -59,6 +61,7 @@ ClpNode::ClpNode (ClpSimplex * model, const ClpNodeStuff * stuff, int depth) :
   fixed_(NULL),
   sequence_(1),
   numberInfeasibilities_(0),
+  depth_(0),
   numberFixed_(0),
   flags_(0),
   maximumFixed_(0),
@@ -82,6 +85,7 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
   int numberColumns = model->numberColumns();
   int numberTotal = numberRows+numberColumns;
   int maximumTotal = maximumRows_+maximumColumns_;
+  depth_ = depth;
   // save stuff
   objectiveValue_ = model->objectiveValue()*model->optimizationDirection();
   estimatedSolution_ = objectiveValue_;
@@ -90,7 +94,8 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
     maximumRows_=CoinMax(maximumRows_,numberRows);
     maximumColumns_=CoinMax(maximumColumns_,numberColumns);
     maximumTotal = maximumRows_+maximumColumns_;
-    factorization_ = new ClpFactorization(*model->factorization());
+    assert (!factorization_);
+    factorization_ = new ClpFactorization(*model->factorization(),numberRows);
     status_ = CoinCopyOfArrayPartial(model->statusArray(),maximumTotal,numberTotal);
     primalSolution_ = CoinCopyOfArrayPartial(model->solutionRegion(),maximumTotal,numberTotal);
     dualSolution_ = CoinCopyOfArrayPartial(model->djRegion(),maximumTotal,numberTotal); //? has duals as well?
@@ -162,8 +167,10 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
 	CoinMemcpyN(model->dualRowSolution(),numberRows,dualSolution_+numberColumns); 
       }
       pivotVariables_ = new int [maximumRows_];
-      if (model->pivotVariable())
+      if (model->pivotVariable()&&model->numberRows()==numberRows)
 	CoinMemcpyN(model->pivotVariable(),numberRows,pivotVariables_); 
+      else
+	CoinFillN(pivotVariables_,numberRows,-1);
     }
   }
   numberFixed_=0;
@@ -177,51 +184,106 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
   int iColumn;
   sequence_=-1;
   double integerTolerance = stuff->integerTolerance_;
-  double mostAway=integerTolerance;
+  double mostAway=0.0;
+  int bestPriority=COIN_INT_MAX;
   sumInfeasibilities_ = 0.0;
   numberInfeasibilities_ = 0;
   int nFix=0;
   double gap = CoinMax(model->dualObjectiveLimit()-objectiveValue_,1.0e-4);
-#define PSEUDO 2
-#if PSEUDO==1
+#define PSEUDO 3
+#if PSEUDO==1||PSEUDO==2
   // Column copy of matrix
   ClpPackedMatrix * matrix = model->clpScaledMatrix();
-  if (!matrix)
+  const double *objective = model->costRegion() ;
+  if (!objective) {
+    objective=model->objective();
+    //if (!matrix)
     matrix = dynamic_cast< ClpPackedMatrix*>(model->clpMatrix());
+  } else if (!matrix) {
+    matrix = dynamic_cast< ClpPackedMatrix*>(model->clpMatrix());
+  }
   const double * element = matrix->getElements();
   const int * row = matrix->getIndices();
   const CoinBigIndex * columnStart = matrix->getVectorStarts();
   const int * columnLength = matrix->getVectorLengths();
-  const double *objective = model->costRegion() ;
   double direction = model->optimizationDirection();
   const double * dual = dualSolution_+numberColumns;
-#elif PSEUDO==2
+#if PSEUDO==2
+  double * activeWeight = new double [numberRows];
+  const double * rowLower = model->rowLower();
+  const double * rowUpper = model->rowUpper();
+  const double * rowActivity = model->primalRowSolution();
+  double tolerance = 1.0e-6;
+  for (int iRow = 0;iRow<numberRows;iRow++) {
+    // could use pi to see if active or activity
+    if (rowActivity[iRow]>rowUpper[iRow]-tolerance
+	||rowActivity[iRow]<rowLower[iRow]+tolerance) {
+      activeWeight[iRow]=0.0;
+    } else {
+      activeWeight[iRow]=-1.0;
+    }
+  }
+  for (int iColumn=0;iColumn<numberColumns;iColumn++) {
+    if (integerType[iColumn]) {
+      double value = solution[iColumn];
+      if (fabs(value-floor(value+0.5))>1.0e-6) {
+	CoinBigIndex start = columnStart[iColumn];
+	CoinBigIndex end = start + columnLength[iColumn];
+	for (CoinBigIndex j=start;j<end;j++) {
+	  int iRow = row[j];
+	  if (activeWeight[iRow]>=0.0)
+	    activeWeight[iRow] += 1.0;
+	}
+      }
+    }
+  }
+  for (int iRow = 0;iRow<numberRows;iRow++) {
+    if (activeWeight[iRow]>0.0) {
+      // could use pi
+      activeWeight[iRow] = 1.0/activeWeight[iRow];
+    } else {
+      activeWeight[iRow]=0.0;
+    }
+  }
+#endif
+#endif
   const double * downPseudo = stuff->downPseudo_;
   const int * numberDown = stuff->numberDown_;
   const int * numberDownInfeasible = stuff->numberDownInfeasible_;
   const double * upPseudo = stuff->upPseudo_;
+  const int * priority = stuff->priority_;
   const int * numberUp = stuff->numberUp_;
   const int * numberUpInfeasible = stuff->numberUpInfeasible_;
-#endif
+  int numberBeforeTrust = stuff->numberBeforeTrust_;
+  int stateOfSearch = stuff->stateOfSearch_;
   int iInteger=0;
+  // weight at 1.0 is max min (CbcBranch was 0.8,0.1) (ClpNode was 0.9,0.9)
+#define WEIGHT_AFTER 0.9
+#define WEIGHT_BEFORE 0.2
+  //Stolen from Constraint Integer Programming book (with epsilon change)
+#define WEIGHT_PRODUCT
+#ifdef WEIGHT_PRODUCT
+  double smallChange = stuff->smallChange_;
+#endif
   for (iColumn=0;iColumn<numberColumns;iColumn++) {
     if (integerType[iColumn]) {
       double value = solution[iColumn];
-      value = max(value,(double) lower[iColumn]);
-      value = min(value,(double) upper[iColumn]);
+      value = CoinMax(value,static_cast<double> (lower[iColumn]));
+      value = CoinMin(value,static_cast<double> (upper[iColumn]));
       double nearest = floor(value+0.5);
       if (fabs(value-nearest)>integerTolerance) {
 	numberInfeasibilities_++;
 	sumInfeasibilities_ += fabs(value-nearest);
-#if PSEUDO==1
+#if PSEUDO==1 || PSEUDO ==2
 	double upValue = 0.0;
 	double downValue = 0.0;
 	double value2 = direction*objective[iColumn];
+	//double dj2=value2;
 	if (value2) {
 	  if (value2>0.0)
-	    upValue += value2;
+	    upValue += 1.5*value2;
 	  else
-	    downValue -= value2;
+	    downValue -= 1.5*value2;
 	}
 	CoinBigIndex start = columnStart[iColumn];
 	CoinBigIndex end = columnStart[iColumn]+columnLength[iColumn];
@@ -230,33 +292,113 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
 	  value2 = -dual[iRow];
 	  if (value2) {
 	    value2 *= element[j];
+	    //dj2 += value2;
+#if PSEUDO==2
+	    assert (activeWeight[iRow]>0.0||fabs(dual[iRow])<1.0e-6);
+	    value2 *= activeWeight[iRow];
+#endif
 	    if (value2>0.0)
 	      upValue += value2;
 	    else
 	      downValue -= value2;
 	  }
 	}
+	//assert (fabs(dj2)<1.0e-4);
+	int nUp=numberUp[iInteger];
+	double upValue2 = (upPseudo[iInteger]/(1.0+nUp));
+	// Extra for infeasible branches
+	if (nUp) {
+	  double ratio = 1.0+static_cast<double>(numberUpInfeasible[iInteger])/
+	    static_cast<double>(nUp);
+	  upValue2 *= ratio;
+	}
+	int nDown = numberDown[iInteger];
+	double downValue2 = (downPseudo[iInteger]/(1.0+nDown));
+	if (nDown) {
+	  double ratio = 1.0+static_cast<double>(numberDownInfeasible[iInteger])/
+	    static_cast<double>(nDown);
+	  downValue2 *= ratio;
+	}
+	//printf("col %d - downPi %g up %g, downPs %g up %g\n",
+	//     iColumn,upValue,downValue,upValue2,downValue2);
+	upValue = CoinMax(0.1*upValue,upValue2);
+	downValue = CoinMax(0.1*downValue,downValue2);
+	//upValue = CoinMax(upValue,1.0e-8);
+	//downValue = CoinMax(downValue,1.0e-8);
 	upValue *= ceil(value)-value;
 	downValue *= value-floor(value);
 	double infeasibility;
-	if (depth>1000)
-	  infeasibility = CoinMax(upValue,downValue)+integerTolerance;
-	else
-	  infeasibility = 0.1*CoinMax(upValue,downValue)+
-	    0.9*CoinMin(upValue,downValue) + integerTolerance;
-#elif PSEUDO==2
-	double upValue = (ceil(value)-value)*(upPseudo[iInteger]/(1.0+numberUp[iInteger]+numberUpInfeasible[iInteger]));
-	double downValue = (value-floor(value))*(downPseudo[iInteger]/(1.0+numberDown[iInteger]+numberDownInfeasible[iInteger]));
+	//if (depth>1000)
+	//infeasibility = CoinMax(upValue,downValue)+integerTolerance;
+	//else
+	if (stateOfSearch<=2) {
+	  // no solution
+	  infeasibility = (1.0-WEIGHT_BEFORE)*CoinMax(upValue,downValue)+
+	    WEIGHT_BEFORE*CoinMin(upValue,downValue) + integerTolerance;
+	} else {
+#ifndef WEIGHT_PRODUCT
+	  infeasibility = (1.0-WEIGHT_AFTER)*CoinMax(upValue,downValue)+
+	    WEIGHT_AFTER*CoinMin(upValue,downValue) + integerTolerance;
+#else
+	  infeasibility = CoinMax(CoinMax(upValue,downValue),smallChange)*
+	    CoinMax(CoinMin(upValue,downValue),smallChange);
+#endif
+	}
+	estimatedSolution_ += CoinMin(upValue2,downValue2);
+#elif PSEUDO==3
+	int nUp=numberUp[iInteger];
+	int nDown = numberDown[iInteger];
+	// Extra 100% for infeasible branches
+	double upValue = (ceil(value)-value)*(upPseudo[iInteger]/
+					      (1.0+nUp));
+	if (nUp) {
+	  double ratio = 1.0+static_cast<double>(numberUpInfeasible[iInteger])/
+	    static_cast<double>(nUp);
+	  upValue *= ratio;
+	}
+	double downValue = (value-floor(value))*(downPseudo[iInteger]/
+						 (1.0+nDown));
+	if (nDown) {
+	  double ratio = 1.0+static_cast<double>(numberDownInfeasible[iInteger])/
+	    static_cast<double>(nDown);
+	  downValue *= ratio;
+	}
+	if (nUp<numberBeforeTrust||nDown<numberBeforeTrust) {
+	  upValue *= 10.0;
+	  downValue *= 10.0;
+	}
+	  
 	double infeasibility;
-	if (depth>1000)
-	  infeasibility = CoinMax(upValue,downValue)+integerTolerance;
-	else
-	  infeasibility = 0.1*CoinMax(upValue,downValue)+
-	    0.9*CoinMin(upValue,downValue) + integerTolerance;
+ 	//if (depth>1000) 
+	//infeasibility = CoinMax(upValue,downValue)+integerTolerance;
+	//else
+	if (stateOfSearch<=2) {
+	  // no solution
+	  infeasibility = (1.0-WEIGHT_BEFORE)*CoinMax(upValue,downValue)+
+	    WEIGHT_BEFORE*CoinMin(upValue,downValue) + integerTolerance;
+	} else {
+#ifndef WEIGHT_PRODUCT
+	  infeasibility = (1.0-WEIGHT_AFTER)*CoinMax(upValue,downValue)+
+	    WEIGHT_AFTER*CoinMin(upValue,downValue) + integerTolerance;
+#else
+	  infeasibility = CoinMax(CoinMax(upValue,downValue),smallChange)*
+	    CoinMax(CoinMin(upValue,downValue),smallChange);
+	  //infeasibility += CoinMin(upValue,downValue)*smallChange;
+#endif
+	}
+	//infeasibility = 0.1*CoinMax(upValue,downValue)+
+	//0.9*CoinMin(upValue,downValue) + integerTolerance;
 	estimatedSolution_ += CoinMin(upValue,downValue);
 #else
 	double infeasibility = fabs(value-nearest);
 #endif
+	assert (infeasibility>0.0);
+	if (priority[iInteger]<bestPriority) {
+	  mostAway=0.0;
+	  bestPriority=priority[iInteger];
+	} else if (priority[iInteger]>bestPriority) {
+	  infeasibility=0.0;
+	}
 	if (infeasibility>mostAway) {
 	  mostAway=infeasibility;
 	  sequence_=iColumn;
@@ -308,6 +450,11 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
       iInteger++;
     }
   }
+  //printf("Choosing %d inf %g pri %d\n",
+  // sequence_,mostAway,bestPriority);
+#if PSEUDO == 2
+  delete [] activeWeight;
+#endif
   if (lower_) {
     // save bounds
     if (iInteger>maximumIntegers_) {
@@ -320,8 +467,8 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
     iInteger=0;
     for (iColumn=0;iColumn<numberColumns;iColumn++) {
       if (integerType[iColumn]) {
-	lower_[iInteger]=(int) lower[iColumn];
-	upper_[iInteger]=(int) upper[iColumn];
+	lower_[iInteger]=static_cast<int> (lower[iColumn]);
+	upper_[iInteger]=static_cast<int> (upper[iColumn]);
 	iInteger++;
       }
     }
@@ -354,7 +501,7 @@ ClpNode::gutsOfConstructor (ClpSimplex * model, const ClpNodeStuff * stuff,
 //-------------------------------------------------------------------
 // Copy constructor 
 //-------------------------------------------------------------------
-ClpNode::ClpNode (const ClpNode & source) 
+ClpNode::ClpNode (const ClpNode & ) 
 {  
   printf("ClpNode copy not implemented\n");
   abort();
@@ -456,9 +603,9 @@ ClpNode::applyNode(ClpSimplex * model, int doBoundsEtc )
     for (int iColumn=0;iColumn<numberColumns;iColumn++) {
       if (integerType[iColumn]) {
 	iInteger++;
-	if (lower_[iInteger]!=(int) lower[iColumn])
+	if (lower_[iInteger]!=static_cast<int> (lower[iColumn]))
 	  model->setColumnLower(iColumn,lower_[iInteger]);
-	if (upper_[iInteger]!=(int) upper[iColumn])
+	if (upper_[iInteger]!=static_cast<int> (upper[iColumn]))
 	  model->setColumnUpper(iColumn,upper_[iInteger]);
       }
     }
@@ -496,10 +643,24 @@ ClpNode::applyNode(ClpSimplex * model, int doBoundsEtc )
     model->setObjectiveValue(objectiveValue_);
   }
 }
+// Choose a new variable
+void 
+ClpNode::chooseVariable(ClpSimplex * , ClpNodeStuff * /*info*/)
+{
+#if 0
+  int way=branchState_.firstBranch;
+  if (branchState_.branch>0)
+    way=1-way;
+  assert (!branchState_.branch);
+  // We need to use pseudo costs to choose a variable
+  int numberColumns = model->numberColumns();
+#endif
+}
 // Fix on reduced costs
 int 
-ClpNode::fixOnReducedCosts(ClpSimplex * model)
+ClpNode::fixOnReducedCosts(ClpSimplex * )
 {
+  
   return 0;
 }
 /* Way for integer variable -1 down , +1 up */
@@ -535,8 +696,10 @@ ClpNode::changeState()
 ClpNodeStuff::ClpNodeStuff () :
   integerTolerance_(1.0e-7),
   integerIncrement_(1.0e-8),
+  smallChange_(1.0e-8),
   downPseudo_(NULL),
   upPseudo_(NULL),
+  priority_(NULL),
   numberDown_(NULL),
   numberUp_(NULL),
   numberDownInfeasible_(NULL),
@@ -549,6 +712,9 @@ ClpNodeStuff::ClpNodeStuff () :
   nBound_(0),
   saveOptions_(0),
   solverOptions_(0),
+  maximumNodes_(0),
+  numberBeforeTrust_(0),
+  stateOfSearch_(0),
   nDepth_(-1),
   nNodes_(0),
   numberNodesExplored_(0),
@@ -564,8 +730,10 @@ ClpNodeStuff::ClpNodeStuff () :
 ClpNodeStuff::ClpNodeStuff (const ClpNodeStuff & rhs) 
   : integerTolerance_(rhs.integerTolerance_),
     integerIncrement_(rhs.integerIncrement_),
+    smallChange_(rhs.smallChange_),
     downPseudo_(NULL),
     upPseudo_(NULL),
+    priority_(NULL),
     numberDown_(NULL),
     numberUp_(NULL),
     numberDownInfeasible_(NULL),
@@ -578,6 +746,9 @@ ClpNodeStuff::ClpNodeStuff (const ClpNodeStuff & rhs)
     nBound_(0),
     saveOptions_(rhs.saveOptions_),
     solverOptions_(rhs.solverOptions_),
+    maximumNodes_(rhs.maximumNodes_),
+    numberBeforeTrust_(rhs.numberBeforeTrust_),
+    stateOfSearch_(rhs.stateOfSearch_),
     nDepth_(rhs.nDepth_),
     nNodes_(rhs.nNodes_),
     numberNodesExplored_(rhs.numberNodesExplored_),
@@ -594,8 +765,10 @@ ClpNodeStuff::operator=(const ClpNodeStuff& rhs)
   if (this != &rhs) {
     integerTolerance_ = rhs.integerTolerance_;
     integerIncrement_ = rhs.integerIncrement_;
+    smallChange_ = rhs.smallChange_;
     downPseudo_ = NULL;
     upPseudo_ = NULL;
+    priority_ = NULL;
     numberDown_ = NULL;
     numberUp_ = NULL;
     numberDownInfeasible_ = NULL;
@@ -608,12 +781,16 @@ ClpNodeStuff::operator=(const ClpNodeStuff& rhs)
     nBound_ = 0;
     saveOptions_ = rhs.saveOptions_;
     solverOptions_ = rhs.solverOptions_;
-    if (nDepth_>=0) {
-      int n = (1<<nDepth_)+1+nDepth_;
+    maximumNodes_ = rhs.maximumNodes_;
+    numberBeforeTrust_ = rhs.numberBeforeTrust_;
+    stateOfSearch_ = rhs.stateOfSearch_;
+    int n = maximumNodes();
+    if (n) {
       for (int i=0;i<n;i++) 
 	delete nodeInfo_[i];
-      delete [] nodeInfo_;
     }
+    delete [] nodeInfo_;
+    nodeInfo_=NULL;
     nDepth_ = rhs.nDepth_;
     nNodes_ = rhs.nNodes_;
     numberNodesExplored_ = rhs.numberNodesExplored_;
@@ -629,6 +806,7 @@ ClpNodeStuff::zap(int type)
   if ((type&1)!=0) {
     downPseudo_ = NULL;
     upPseudo_ = NULL;
+    priority_ = NULL;
     numberDown_ = NULL;
     numberUp_ = NULL;
     numberDownInfeasible_ = NULL;
@@ -643,6 +821,9 @@ ClpNodeStuff::zap(int type)
     nBound_ = 0;
     saveOptions_ = 0;
     solverOptions_ = 0;
+    maximumNodes_ = 0;
+    numberBeforeTrust_ = 0;
+    stateOfSearch_ = 0;
     nDepth_ = -1;
     nNodes_ = 0;
     presolveType_ = 0;
@@ -658,20 +839,54 @@ ClpNodeStuff::~ClpNodeStuff ()
 {
   delete [] downPseudo_;
   delete [] upPseudo_;
+  delete [] priority_;
   delete [] numberDown_;
   delete [] numberUp_;
   delete [] numberDownInfeasible_;
   delete [] numberUpInfeasible_;
-  if (nDepth_>=0) {
-    int n = (1<<nDepth_)+1+nDepth_;
+  int n = maximumNodes();
+  if (n) {
     for (int i=0;i<n;i++) 
       delete nodeInfo_[i];
-    delete [] nodeInfo_;
   }
+  delete [] nodeInfo_;
+#ifdef CLP_INVESTIGATE
+  // Should be NULL - find out why not?
+  assert (!saveCosts_);
+#endif
+    delete [] saveCosts_;
+}
+// Return maximum number of nodes
+int 
+ClpNodeStuff::maximumNodes() const
+{
+  int n=0;
+#if 0
+  if (nDepth_!=-1) {
+    if ((solverOptions_&32)==0) 
+      n = (1<<nDepth_);
+    else if (nDepth_) 
+      n = 1;
+  }
+  assert (n==maximumNodes_-(1+nDepth_)||n==0);
+#else
+  if (nDepth_!=-1) {
+    n = maximumNodes_-(1+nDepth_);
+    assert (n>0);
+  }
+#endif
+  return n;
+}
+// Return maximum space for nodes
+int 
+ClpNodeStuff::maximumSpace() const
+{
+  return maximumNodes_;
 }
 /* Fill with pseudocosts */
 void 
 ClpNodeStuff::fillPseudoCosts(const double * down, const double * up, 
+			      const int * priority,
 			      const int * numberDown, const int * numberUp,
 			      const int * numberDownInfeasible, 
 			      const int * numberUpInfeasible,
@@ -679,12 +894,14 @@ ClpNodeStuff::fillPseudoCosts(const double * down, const double * up,
 {
   delete [] downPseudo_;
   delete [] upPseudo_;
+  delete [] priority_;
   delete [] numberDown_;
   delete [] numberUp_;
   delete [] numberDownInfeasible_;
   delete [] numberUpInfeasible_;
   downPseudo_ = CoinCopyOfArray(down,number);
   upPseudo_ = CoinCopyOfArray(up,number);
+  priority_ = CoinCopyOfArray(priority,number);
   numberDown_ = CoinCopyOfArray(numberDown,number);
   numberUp_ = CoinCopyOfArray(numberUp,number);
   numberDownInfeasible_ = CoinCopyOfArray(numberDownInfeasible,number);
@@ -692,10 +909,10 @@ ClpNodeStuff::fillPseudoCosts(const double * down, const double * up,
   // scale
   for (int i=0;i<number;i++) {
     int n;
-    n = numberDown_[i]+numberDownInfeasible_[i];
+    n = numberDown_[i];
     if (n)
       downPseudo_[i] *= n;
-    n = numberUp_[i]+numberUpInfeasible_[i];
+    n = numberUp_[i];
     if (n)
       upPseudo_[i] *= n;
   }
@@ -704,16 +921,16 @@ ClpNodeStuff::fillPseudoCosts(const double * down, const double * up,
 void 
 ClpNodeStuff::update(int way,int sequence,double change,bool feasible)
 {
+  assert (numberDown_[sequence]>=numberDownInfeasible_[sequence]);
+  assert (numberUp_[sequence]>=numberUpInfeasible_[sequence]);
   if (way<0) {
-    if (feasible)
-      numberDown_[sequence]++;
-    else
+    numberDown_[sequence]++;
+    if (!feasible)
       numberDownInfeasible_[sequence]++;
     downPseudo_[sequence] += CoinMax(change,1.0e-12);
   } else {
-    if (feasible)
-      numberUp_[sequence]++;
-    else
+    numberUp_[sequence]++;
+    if (!feasible)
       numberUpInfeasible_[sequence]++;
     upPseudo_[sequence] += CoinMax(change,1.0e-12);
   }
